@@ -88,7 +88,7 @@ XftDrawDepth (XftDraw *draw)
 	Window		    root;
 	int		    x, y;
 	unsigned int	    width, height, borderWidth, depth;
-	if (XGetGeometry (draw->dpy, draw->drawable, 
+	if (XGetGeometry (draw->dpy, draw->drawable,
 			  &root, &x, &y, &width, &height,
 			  &borderWidth, &depth))
 	    draw->depth = depth;
@@ -104,7 +104,7 @@ XftDrawBitsPerPixel (XftDraw	*draw)
 	XPixmapFormatValues *formats;
 	int		    nformats;
 	unsigned int	    depth;
-	
+
 	if ((depth = XftDrawDepth (draw)) &&
 	    (formats = XListPixmapFormats (draw->dpy, &nformats)))
 	{
@@ -112,9 +112,9 @@ XftDrawBitsPerPixel (XftDraw	*draw)
 
 	    for (i = 0; i < nformats; i++)
 	    {
-		if (formats[i].depth == depth)
+		if ((unsigned) formats[i].depth == depth)
 		{
-		    draw->bits_per_pixel = formats[i].bits_per_pixel;
+		    draw->bits_per_pixel = (unsigned)formats[i].bits_per_pixel;
 		    break;
 		}
 	    }
@@ -132,10 +132,10 @@ XftDrawCreate (Display   *dpy,
 {
     XftDraw	*draw;
 
-    draw = (XftDraw *) malloc (sizeof (XftDraw));
+    draw = malloc (sizeof (XftDraw));
     if (!draw)
 	return NULL;
-    
+
     draw->dpy = dpy;
     draw->drawable = drawable;
     draw->screen = _XftDrawScreen (dpy, drawable, visual);
@@ -158,7 +158,7 @@ XftDrawCreateBitmap (Display	*dpy,
 {
     XftDraw	*draw;
 
-    draw = (XftDraw *) malloc (sizeof (XftDraw));
+    draw = malloc (sizeof (XftDraw));
     if (!draw)
 	return NULL;
     draw->dpy = dpy;
@@ -184,13 +184,13 @@ XftDrawCreateAlpha (Display *dpy,
 {
     XftDraw	*draw;
 
-    draw = (XftDraw *) malloc (sizeof (XftDraw));
+    draw = malloc (sizeof (XftDraw));
     if (!draw)
 	return NULL;
     draw->dpy = dpy;
     draw->drawable = (Drawable) pixmap;
     draw->screen = _XftDrawScreen (dpy, pixmap, NULL);
-    draw->depth = depth;
+    draw->depth = (unsigned)depth;
     draw->bits_per_pixel = 0;	/* don't find out until we need it */
     draw->visual = NULL;
     draw->colormap = 0;
@@ -216,9 +216,9 @@ _XftDrawFormat (XftDraw	*draw)
 	XRenderPictFormat   pf;
 
 	pf.type = PictTypeDirect;
-	pf.depth = XftDrawDepth (draw);
+	pf.depth = (int)XftDrawDepth (draw);
 	pf.direct.alpha = 0;
-	pf.direct.alphaMask = (1 << pf.depth) - 1;
+	pf.direct.alphaMask = (short)((1 << pf.depth) - 1);
 	return XRenderFindFormat (draw->dpy,
 				  (PictFormatType|
 				   PictFormatDepth|
@@ -301,9 +301,9 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
     int		    i;
     XftColor	    bitmapColor;
 
-    if (!info)
+    if (!info || !info->solidFormat)
 	return 0;
-    
+
     /*
      * Monochrome targets require special handling; the PictOp controls
      * the color, and the color must be opaque
@@ -322,10 +322,10 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
      */
     for (i = 0; i < XFT_NUM_SOLID_COLOR; i++)
     {
-	if (info->colors[i].pict && 
+	if (info->colors[i].pict &&
 	    info->colors[i].screen == draw->screen &&
-	    !memcmp ((void *) &color->color, 
-		     (void *) &info->colors[i].color,
+	    !memcmp ((const void *) &color->color,
+		     (const void *) &info->colors[i].color,
 		     sizeof (XRenderColor)))
 	    return info->colors[i].pict;
     }
@@ -333,39 +333,52 @@ XftDrawSrcPicture (XftDraw *draw, _Xconst XftColor *color)
      * Pick one to replace at random
      */
     i = (unsigned int) rand () % XFT_NUM_SOLID_COLOR;
-    /*
-     * Recreate if it was for the wrong screen
-     */
-    if (info->colors[i].screen != draw->screen && info->colors[i].pict)
-    {
-	XRenderFreePicture (dpy, info->colors[i].pict);
-	info->colors[i].pict = 0;
+
+    if (info->hasSolid) {
+	/*
+	 * Free any existing entry
+	 */
+	if (info->colors[i].pict)
+	    XRenderFreePicture (dpy, info->colors[i].pict);
+	/*
+	 * Create picture
+	 */
+	info->colors[i].pict = XRenderCreateSolidFill (draw->dpy, &color->color);
+    } else {
+	if (info->colors[i].screen != draw->screen && info->colors[i].pict)
+	{
+	    XRenderFreePicture (dpy, info->colors[i].pict);
+	    info->colors[i].pict = 0;
+	}
+	/*
+	 * Create picture if necessary
+	 */
+	if (!info->colors[i].pict)
+	{
+	    Pixmap			    pix;
+	    XRenderPictureAttributes    pa;
+
+	    pix = XCreatePixmap (dpy, RootWindow (dpy, draw->screen), 1, 1,
+				 (unsigned)info->solidFormat->depth);
+	    pa.repeat = True;
+	    info->colors[i].pict = XRenderCreatePicture (draw->dpy,
+							 pix,
+							 info->solidFormat,
+							 CPRepeat, &pa);
+	    XFreePixmap (dpy, pix);
+	}
+	/*
+	 * Set to the new color
+	 */
+	info->colors[i].color = color->color;
+	info->colors[i].screen = draw->screen;
+	XRenderFillRectangle (dpy, PictOpSrc,
+			      info->colors[i].pict,
+			      &color->color, 0, 0, 1, 1);
     }
-    /*
-     * Create picture if necessary
-     */
-    if (!info->colors[i].pict)
-    {
-	Pixmap			    pix;
-        XRenderPictureAttributes    pa;
-	
-	pix = XCreatePixmap (dpy, RootWindow (dpy, draw->screen), 1, 1,
-			     info->solidFormat->depth);
-	pa.repeat = True;
-	info->colors[i].pict = XRenderCreatePicture (draw->dpy,
-						     pix,
-						     info->solidFormat,
-						     CPRepeat, &pa);
-	XFreePixmap (dpy, pix);
-    }
-    /*
-     * Set to the new color
-     */
     info->colors[i].color = color->color;
     info->colors[i].screen = draw->screen;
-    XRenderFillRectangle (dpy, PictOpSrc,
-			  info->colors[i].pict,
-			  &color->color, 0, 0, 1, 1);
+
     return info->colors[i].pict;
 }
 
@@ -391,7 +404,7 @@ _XftDrawRenderPrepare (XftDraw	*draw)
 	format = _XftDrawFormat (draw);
 	if (!format)
 	    return FcFalse;
-	
+
 	if (draw->subwindow_mode == IncludeInferiors)
 	{
 	    pa.subwindow_mode = IncludeInferiors;
@@ -454,7 +467,7 @@ _XftDrawCorePrepare (XftDraw *draw, _Xconst XftColor *color)
     XSetForeground (draw->dpy, draw->core.gc, color->pixel);
     return FcTrue;
 }
-			
+
 _X_EXPORT Picture
 XftDrawPicture (XftDraw *draw)
 {
@@ -479,7 +492,7 @@ XftDrawGlyphs (XftDraw		*draw,
     if (font->format)
     {
 	Picture	    src;
-	
+
 	if (_XftDrawRenderPrepare (draw) &&
 	    (src = XftDrawSrcPicture (draw, color)))
 	    XftGlyphRender (draw->dpy, _XftDrawOp (draw, color),
@@ -497,7 +510,7 @@ _X_EXPORT void
 XftDrawString8 (XftDraw		    *draw,
 		_Xconst XftColor    *color,
 		XftFont		    *pub,
-		int		    x, 
+		int		    x,
 		int		    y,
 		_Xconst FcChar8	    *string,
 		int		    len)
@@ -507,12 +520,12 @@ XftDrawString8 (XftDraw		    *draw,
 
     if (XftDebug () & XFT_DBG_DRAW)
 	printf ("DrawString \"%*.*s\"\n", len, len, string);
-    
+
     if (len <= NUM_LOCAL)
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (FT_UInt));
+	glyphs = AllocUIntArray (len);
 	if (!glyphs)
 	    return;
     }
@@ -535,17 +548,20 @@ XftDrawString16 (XftDraw	    *draw,
     FT_UInt	    *glyphs, glyphs_local[NUM_LOCAL];
     int		    i;
 
+    if (len <= 0)
+	return;
+
     if (len <= NUM_LOCAL)
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (FT_UInt));
+	glyphs = AllocUIntArray (len);
 	if (!glyphs)
 	    return;
     }
     for (i = 0; i < len; i++)
 	glyphs[i] = XftCharIndex (draw->dpy, pub, string[i]);
-    
+
     XftDrawGlyphs (draw, color, pub, x, y, glyphs, len);
     if (glyphs != glyphs_local)
 	free (glyphs);
@@ -563,17 +579,20 @@ XftDrawString32 (XftDraw	    *draw,
     FT_UInt	    *glyphs, glyphs_local[NUM_LOCAL];
     int		    i;
 
+    if (len <= 0)
+	return;
+
     if (len <= NUM_LOCAL)
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (FT_UInt));
+	glyphs = AllocUIntArray (len);
 	if (!glyphs)
 	    return;
     }
     for (i = 0; i < len; i++)
 	glyphs[i] = XftCharIndex (draw->dpy, pub, string[i]);
-    
+
     XftDrawGlyphs (draw, color, pub, x, y, glyphs, len);
     if (glyphs != glyphs_local)
 	free (glyphs);
@@ -583,7 +602,7 @@ _X_EXPORT void
 XftDrawStringUtf8 (XftDraw	    *draw,
 		   _Xconst XftColor *color,
 		   XftFont	    *pub,
-		   int		    x, 
+		   int		    x,
 		   int		    y,
 		   _Xconst FcChar8  *string,
 		   int		    len)
@@ -594,6 +613,9 @@ XftDrawStringUtf8 (XftDraw	    *draw,
     int		    l;
     int		    size;
 
+    if (len <= 0)
+	return;
+
     i = 0;
     glyphs = glyphs_local;
     size = NUM_LOCAL;
@@ -601,14 +623,14 @@ XftDrawStringUtf8 (XftDraw	    *draw,
     {
 	if (i == size)
 	{
-	    glyphs_new = malloc (size * 2 * sizeof (FT_UInt));
+	    glyphs_new = AllocUIntArray (size * 2);
 	    if (!glyphs_new)
 	    {
 		if (glyphs != glyphs_local)
 		    free (glyphs);
 		return;
 	    }
-	    memcpy (glyphs_new, glyphs, size * sizeof (FT_UInt));
+	    memcpy (glyphs_new, glyphs, (size_t)size * sizeof (FT_UInt));
 	    size *= 2;
 	    if (glyphs != glyphs_local)
 		free (glyphs);
@@ -639,6 +661,9 @@ XftDrawStringUtf16 (XftDraw		*draw,
     int		    l;
     int		    size;
 
+    if (len <= 0)
+	return;
+
     i = 0;
     glyphs = glyphs_local;
     size = NUM_LOCAL;
@@ -646,14 +671,14 @@ XftDrawStringUtf16 (XftDraw		*draw,
     {
 	if (i == size)
 	{
-	    glyphs_new = malloc (size * 2 * sizeof (FT_UInt));
+	    glyphs_new = AllocUIntArray (size * 2);
 	    if (!glyphs_new)
 	    {
 		if (glyphs != glyphs_local)
 		    free (glyphs);
 		return;
 	    }
-	    memcpy (glyphs_new, glyphs, size * sizeof (FT_UInt));
+	    memcpy (glyphs_new, glyphs, (size_t)size * sizeof (FT_UInt));
 	    size *= 2;
 	    if (glyphs != glyphs_local)
 		free (glyphs);
@@ -742,11 +767,14 @@ XftDrawCharSpec (XftDraw		*draw,
     XftGlyphSpec    *glyphs, glyphs_local[NUM_LOCAL];
     int		    i;
 
+    if (len <= 0)
+	return;
+
     if (len <= NUM_LOCAL)
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (XftGlyphSpec));
+	glyphs = AllocGlyphSpecArray (len);
 	if (!glyphs)
 	    return;
     }
@@ -771,11 +799,14 @@ XftDrawCharFontSpec (XftDraw			*draw,
     XftGlyphFontSpec	*glyphs, glyphs_local[NUM_LOCAL];
     int			i;
 
+    if (len <= 0)
+	return;
+
     if (len <= NUM_LOCAL)
 	glyphs = glyphs_local;
     else
     {
-	glyphs = malloc (len * sizeof (XftGlyphFontSpec));
+	glyphs = AllocGlyphFontSpecArray (len);
 	if (!glyphs)
 	    return;
     }
@@ -795,7 +826,7 @@ XftDrawCharFontSpec (XftDraw			*draw,
 _X_EXPORT void
 XftDrawRect (XftDraw		*draw,
 	     _Xconst XftColor	*color,
-	     int		x, 
+	     int		x,
 	     int		y,
 	     unsigned int	width,
 	     unsigned int	height)
@@ -825,9 +856,9 @@ XftDrawSetClip (XftDraw	*draw,
      */
     if (!r && draw->clip_type == XftClipTypeNone)
 	return True;
-    
-    if (r && 
-	draw->clip_type == XftClipTypeRegion && 
+
+    if (r &&
+	draw->clip_type == XftClipTypeRegion &&
 	XEqualRegion (r, draw->clip.region))
     {
 	return True;
@@ -862,7 +893,7 @@ XftDrawSetClip (XftDraw	*draw,
     case XftClipTypeNone:
 	break;
     }
-    
+
     /*
      * Set the clip
      */
@@ -916,7 +947,7 @@ XftDrawSetClipRectangles (XftDraw		*draw,
 	draw->clip.rect->n == n &&
 	(n == 0 || (draw->clip.rect->xOrigin == xOrigin &&
 		    draw->clip.rect->yOrigin == yOrigin)) &&
-	!memcmp (XftClipRects (draw->clip.rect), rects, n * sizeof (XRectangle)))
+	!memcmp (XftClipRects (draw->clip.rect), rects, (size_t)n * sizeof (XRectangle)))
     {
 	return True;
     }
@@ -924,14 +955,14 @@ XftDrawSetClipRectangles (XftDraw		*draw,
     /*
      * Duplicate the region so future changes can be short circuited
      */
-    new = malloc (sizeof (XftClipRect) + n * sizeof (XRectangle));
+    new = malloc (sizeof (XftClipRect) + (size_t)n * sizeof (XRectangle));
     if (!new)
 	return False;
 
     new->n = n;
     new->xOrigin = xOrigin;
     new->yOrigin = yOrigin;
-    memcpy (XftClipRects (new), rects, n * sizeof (XRectangle));
+    memcpy (XftClipRects (new), rects, (size_t)n * sizeof (XRectangle));
 
     /*
      * Destroy existing clip
@@ -946,7 +977,7 @@ XftDrawSetClipRectangles (XftDraw		*draw,
     case XftClipTypeNone:
 	break;
     }
-    
+
     /*
      * Set the clip
      */
@@ -986,7 +1017,7 @@ XftDrawSetSubwindowMode (XftDraw *draw, int mode)
 	XRenderPictureAttributes    pa;
 
 	pa.subwindow_mode = mode;
-	XRenderChangePicture (draw->dpy, draw->render.pict, 
+	XRenderChangePicture (draw->dpy, draw->render.pict,
 			      CPSubwindowMode, &pa);
     }
     if (draw->core.gc)
